//------------------------------------------------------------------------------------------
// PIC Beacon Keyer  -   first attempt with the 'free' version of CC5X .
// 
// File:   c:\cproj\BeaconControl\Main1.c 
// Date:   2004-02-12 (YYYY-MM-DD)
// Author: Wolfgang Buescher (DL4YHF), Spenge, Germany .
// Tool:   MPLAB 6.40 (PIC development IDE + Simulator) by Microchip, www.microchip.com .
//         CC5X free edition
//            (c) B Knudsen Data, Trondheim, Norway, 1992 - 2003, www.bknd.com .
//
// Notes:
//   To any programmer who must // wants to  maintain this project
//    - Don't try to give the program a better look ! Don't use loops wherever possible !
//    - Keep an eye on the code size; CC5X "free" allows a maximum of 1024 Words only   !
//    - Don't use VARIABLE array indexes wherever possible, only use constants instead  !
//------------------------------------------------------------------------------------------


// First define the device. It can be overridden by a similar
// command line option (-p16F877). Remember to supply the path to
// the valid header file (option -Ic:\cc5x or similar) if this is
// not located in the current directory /**/
// #include "16F628.h"  // must(!) not be included (?!?)
#pragma chip PIC16F628
#define __16F628


// Define the PIC's configuration word, see PIC datasheet..
//   Oscillator: 0b100=INTRC, usually 4MHz, here 37kHz selected via PCON.OSCF
#pragma config |= 0b11110000000000  // bit13..10 = 1111 -> no program code protection
#pragma config |= 0b00000100000000  // bit8 = 1  -> no data code protection
#pragma config &= 0b11111101111111  // bit7 = 0  -> no LOW VOLTAGE progging, RB4 usable for I/O
#pragma config &= 0b11111111011111  // bit5 = 0  -> no MCLR,  RA5 usable for I/O 
#pragma config PWRTE=on, WDTE=off, FOSC=0b100, BODEN=off


// Define the PIC's indentifier locations. May be read unscrambled.
#pragma config ID[0] = 0x01
#pragma config ID[1] = 0x02
#pragma config ID[2] = 0x03
#pragma config ID[3] = 0x04


/* HEADER FILES: define IO, variables, constants, prototypes, etc. */
// #include "math16.h"
// #include "myheader.h"
// #include "project\iodef.h"


// Some 'compatibility definitions' ...
#ifndef BYTE
  #define BYTE uns8
#endif

// I/O port definitions. Compatible with DL4YHF's PIC-QRP-KEYER wherever possible.
//
// Note: PORTA has no internal pull-ups. Unused INPUTS should be tied to Vcc
//       with an external pull-up to avoid floating, which draws add'l current.
//       For this reason, port A is used as OUTPUT as far as possible.
bit IOP_poti_1     @ PORTA.0; // only if poti is POLLED (NO IRQ!!)
bit IOP_key_output @ PORTA.1; // modulation output:    1=HF on,  0=HF off
bit IOP_tx_control @ PORTA.2; // transmitter control:  1=TX,     0=RX
bit IOP_audio_out  @ PORTA.3; // sidetone (piezo speaker, ~650 Hz audio here)
bit IOP_thermo_sens @ PORTA.4; // reserved for monoflop with NTC+cap as thermo sensor

// Note: PORTB will have its internal pullups enabled.
//       (so there's no need to tie unused pins to GND or VCC.)
bit IOP_reverve1   @ PORTB.0; // "for future use" ....  ..
bit IOP_signal_led @ PORTB.2; // "Signal"-LED instead of piezo speaker
bit IOP_hand_key   @ PORTB.3; // from NORCALs keyer. not used yet.
bit IOP_dot        @ PORTB.4; // the inputs of PORTB are used for wake-up    
bit IOP_dash       @ PORTB.5;
bit IOP_msg_key1   @ PORTB.6; // DL4YHF's "MESSAGE 1" is NORCALs "MSG"
bit IOP_msg_key2   @ PORTB.7; // DL4YHF's "MESSAGE 2" is NORCALs "FREQ"


//======================================================================
// Constant definitions
//======================================================================


 // Storage format of "spaces", "control characters" and "transmittable CW letters"
 // ---------------------------------------------------------------------------------
 //  The most significant bits define the code type.  
 //     Bit7: 1 = "this is SPACE (=pause) or CONTROL character", in this case: 
 //        Bit6,Bit5 = control character type:
 //        0x = SPACE (=pause,     bits5..0 contain the length in DOTS (max.63)
 //        10 = extra long "dash", bits4..0 contain the length in DOTS (max.31)
 //        11 = future reserve,    bits4..0 contain 31 possible codes
 //     Bit7: 0 = "this is a transmittable character (not SPACE or CONTROL)".
 //        Bits6..0 contain a maximum of 6 dashes or dots,
 //             with leading 1="Startbit" before the "dash/dot-matrix",
 //             dash/dot-matrix is in Bit0..max.Bit5: 
 //                  0=dot 1=dash
 //             Bit0 always contains the LAST TRANSMITTED dash/dot.
 // See definitions below for some examples how CW letters are stored in memory.
#define CW_CHR_0    0b00111111  // "0" 
#define CW_CHR_1    0b00101111  // "1"
#define CW_CHR_2    0b00100111  // "2"
#define CW_CHR_3    0b00100011  // "3"
#define CW_CHR_4    0b00100001  // "4"
#define CW_CHR_5    0b00100000  // "5"
#define CW_CHR_6    0b00110000  // "6"
#define CW_CHR_7    0b00111000  // "7"
#define CW_CHR_8    0b00111100  // "8"
#define CW_CHR_9    0b00111110  // "9"

#define CW_CHR_a    0b00000101  // 'A'
#define CW_CHR_b    0b00011000  // 'B'
#define CW_CHR_c    0b00011010  // 'C'
#define CW_CHR_d    0b00001100  // 'D'
#define CW_CHR_e    0b00000010  // 'E'
#define CW_CHR_f    0b00010010  // 'F'
#define CW_CHR_g    0b00001110  // 'G'
#define CW_CHR_h    0b00010000  // 'H'
#define CW_CHR_i    0b00000100  // 'I'
#define CW_CHR_j    0b00010111  // 'J'
#define CW_CHR_k    0b00001101  // 'K'
#define CW_CHR_l    0b00010100  // 'L'
#define CW_CHR_m    0b00000111  // 'M'
#define CW_CHR_n    0b00000110  // 'N'
#define CW_CHR_o    0b00001111  // 'O'
#define CW_CHR_p    0b00010110  // 'P'
#define CW_CHR_q    0b00011101  // 'Q'
#define CW_CHR_r    0b00001010  // 'R'
#define CW_CHR_s    0b00001000  // 'S'
#define CW_CHR_t    0b00000011  // 'T'
#define CW_CHR_u    0b00001001  // 'U'
#define CW_CHR_v    0b00010001  // 'V'
#define CW_CHR_w    0b00001011  // 'W'
#define CW_CHR_x    0b00011001  // 'X'
#define CW_CHR_y    0b00011011  // 'Y'
#define CW_CHR_z    0b00011100  // 'Z'

#define CW_CHR_SEPAR 0b00110001  // '='  (-...-)  this is the "official" separator
#define CW_CHR_SEPA2 0b01100001  // '-'  (-....-) but this is also frequenty used
#define CW_CHR_POINT 0b01010101  // '.'  (-.-.-.)
#define CW_CHR_SLASH 0b00110010  // '/'  (-..-.)
#define CW_CHR_QUESTN 0b01001100  // '?'  (..--..)
#define CW_CHR_AR    0b00101010  // 'AR' (.-.-.)
#define CW_CHR_SK    0b01000101  // 'SK' (...-.-)
#define CW_CHR_KA    0b00110101  // 'KA' (-.-.-)
#define CW_CHR_KN    0b00110110  // 'KN' (-.--.)
#define CW_CHR_EOM   0b01011111  // 'EOM' (.-----) used for "partitions" of msg
#define CW_CHR_NNN   0b01101010  // 'NNN' (-.-.-.) replaced by serial number
#define CW_CHR_ANN   0b01011010  // 'ANN' (.--.-.) Advance to Next Number
#define CW_CHR_SPACE 0b10000011  // ' ' (pause length in "dots" in lower 5 bits, here : standard space )

#define CW_CHR_TUNE1 0b11011111  // extra long "dash", bits4..0 contain the length

// How many audio periods per morse-dot ? 
// A trial-and-error test showed '70' audio periods (640 Hz)
//   gave a CW speed comparable to many VHF beacons.
// Caution: the following constant must be less than 255/3 = 85
#define C_AUDIO_CYCLES_PER_DOT 70


// DEFINING RAM VARIABLES

// Keyer Variables
BYTE KY_mem_index;  // Keyer memory index for record + replay



// Include MATH LIBRARIES after interrupt definition
// #include "math24f.h"   // 24 bit basic floating point operations
// #include "math24lb.h"  // 24 bit math function library


// CODEPAGE definitions : none !


// Initial EEPROM data  ( PIC16F628: 128 byte EEPROM)
#define CDATA_START  0x2100
#pragma cdata[CDATA_START]  // Start of cdata block. Note the LOW-HIGH order of BYTES in a WORD ..
        // Default message text in EEPROM ..
        // '\0' must be added in strings due to 14 bit compression
#pragma cdata[] = ' ','d','b','0','u','b','i'
#pragma cdata[] = ' ','j','o','4','2','g','e',' '
#pragma cdata[] = ' ',' ','p','w','r',' ','2','4','0',' ','m','w',' ','e','r','p',' ' 
#pragma cdata[] = ' ',' ','a','n','t',' ','o','m','n','i',' '
#pragma cdata[] = ' ','A',' ',' ','T',' '
#pragma cdata[] = ' ','d','b','0','u','b','i',' ',' ','T'
#pragma cdata[] = ' ','d','b','0','u','b','i',' ',' ','T'
#pragma cdata[] = ' ','d','b','0','u','b','i',' ',' ','T'
#pragma cdata[] = 0   



//----------------------------------------------------------------------------
BYTE Ascii_to_CW(BYTE bAscii)
{
 // binary code patterns to convert ASCII into morse code patterns.
 // table INDEX is ASCII,  table content is 8-BIT-CW-PATTERN .
 static const BYTE bDigits[] =  // [0...9] :
  { CW_CHR_0, CW_CHR_1, CW_CHR_2, CW_CHR_3, CW_CHR_4, 
    CW_CHR_5, CW_CHR_6, CW_CHR_7, CW_CHR_8, CW_CHR_9 };  
 static const BYTE bLetters[] = // [a...z] :
  { CW_CHR_a, CW_CHR_b, CW_CHR_c, CW_CHR_d, CW_CHR_e, 
    CW_CHR_f, CW_CHR_g, CW_CHR_h, CW_CHR_i, CW_CHR_j, 
    CW_CHR_k, CW_CHR_l, CW_CHR_m, CW_CHR_n, CW_CHR_o, 
    CW_CHR_p, CW_CHR_q, CW_CHR_r, CW_CHR_s, CW_CHR_t, 
    CW_CHR_u, CW_CHR_v, CW_CHR_w, CW_CHR_x, CW_CHR_y, 
    CW_CHR_z };  


 // The assembled code should be a computed goto (widh ADDWF PCL,1 and a lot of RETLW's ) .
 // The sequence '	MOVWF bValue ;  MOVF  bValue,W ' generated by the free compiler is rubbish
 //          but there is no easy workaround.  Anyway, it's not an error !
 if( bAscii>='0' && bAscii<='9')    // digits
   { W = bAscii-'0';
     return bDigits[W];
   }
 if( bAscii>='a' && bAscii<='z')    // normal letters: lower case
   { W = bAscii-'a';
     return bLetters[W];
   }
 // special characters from case list :
 switch( bAscii )
   {
     case ' ': return CW_CHR_SPACE;  // ' ' (pause length in "dots" will be added)
     case '=': return CW_CHR_SEPAR;  // '='  (-...-)  this is the "official" separator
     case '-': return CW_CHR_SEPA2;  // '-'  (-....-) but this is also frequenty used
     case '.': return CW_CHR_POINT;  // '.'  (-.-.-.)
     case '/': return CW_CHR_SLASH;  // '/'  (-..-.)
     case '?': return CW_CHR_QUESTN; // '?'  (..--..)
     case 'A': return CW_CHR_AR;     // 'AR' (.-.-.)
     case 'S': return CW_CHR_SK;     // 'SK' (...-.-)
     case 'K': return CW_CHR_KA;     // 'KA' (-.-.-)
     case 'N': return CW_CHR_KN;     // 'KN' (-.--.)
     case 'E': return CW_CHR_EOM;    // 'EOM' (.-----) used for "partitions" of msg
     case 'C': return CW_CHR_NNN;    // 'C' transmit COUNTER alias serial number
     case 'I': return CW_CHR_ANN;    // 'I' increment COUNTER alias serial number 
     case 'T': return CW_CHR_TUNE1;  // TUNE TONE #1 (31 dash times)
     default: return CW_CHR_QUESTN; 
   }

 return 0x00;
} // end seven_segment_pattern()


//----------------------------------------------------------------------------
void SendTone(char W)
  // Produces a sidetone for <W> DOT times .
{
  BYTE bDots;
  BYTE bAudioCycles;

  bDots = W;

  // How many audio periods per morse-dot ? 
  // At relatively low speed, 250 milliseconds per dot.
  // A good audio note -say 600 Hz- these are 150 cycles.
  bAudioCycles = C_AUDIO_CYCLES_PER_DOT; // initial guess


  if(bDots==3)
   { // for 3 dots or less, do NOT perform a "dot loop" 
     // to avoid audible kinks in the sidetone :
     bDots = 1;
     bAudioCycles = (3*C_AUDIO_CYCLES_PER_DOT); // (a LITERAL, no MULTIPLY)
   } // end if(bDots==3)


  while(bDots)     // Repeat THIS loop only if really necessary because 
   { bDots--;      // it causes a "kink" in the software-generated sidetone !


     //    How many loops per 'audio period' ?
     //    PIC is clocked with 37 kHz, 4 cycles per instruction, 
     //    so 9 .. 12 instructions per full audio period are ok !
     while(bAudioCycles)
      {  --bAudioCycles;
        IOP_audio_out = 1; // sidetone output HIGH
        nop();  // wait while sidetone-output HIGH
        IOP_audio_out = 0; // sidetone output LOW
        // not necessary do wait while sidetone-output LOW,
        // the time for the "C" loop will do the rest !
      } // end while(bAudioCycles)

     // reload cycle timer just in case bDots is still > 0
     bAudioCycles = C_AUDIO_CYCLES_PER_DOT;  
   } // end while(bDots)
} // end SendTone()


//----------------------------------------------------------------------------
void SendPause(char W)
  // Delay time for <W> DOT times .
{
  BYTE bDots;
  BYTE bAudioCycles;

  bDots = W;

  while(bDots)     // 
   { bDots--;

     bAudioCycles = C_AUDIO_CYCLES_PER_DOT;

     while(bAudioCycles)
      {  --bAudioCycles;
        nop();  // compensate for sidetone output command
        nop();  // wait while sidetone-output HIGH
        nop();  // compensate for sidetone output command
        // not necessary do wait while sidetone-output LOW,
        // the time for the "C" loop will do the rest !
      } // end while(bAudioCycles)

   } // end while(bDots)
} // end SendPause()


//----------------------------------------------------------------------------
void TxKeyDown(void)
{
  IOP_key_output = 1;  // modulation "on"
} // end TxKeyDown()

void TxKeyUp(void)
{
  IOP_key_output = 0;  // modulation "off"
} // end TxKeyUp()




//----------------------------------------------------------------------------
void SendMorse(BYTE bPattern)
 // Send in morse code.
 // bPattern: "spaces", "control characters" and "transmittable CW letters"
 //  The most significant bits define the code type.  
 //     Bit7: 1 = "this is SPACE (=pause) or CONTROL character", in this case: 
 //        Bit6,Bit5 = control character type:
 //        0x = SPACE (=pause,     bits5..0 contain the length in DOTS (max.63)
 //        10 = extra long "dash", bits4..0 contain the length in DOTS (max.31)
 //        11 = future reserve,    bits4..0 contain 31 possible codes
 //     Bit7: 0 = "this is a transmittable character (not SPACE or CONTROL)".
 //        Bits6..0 contain a maximum of 6 dashes or dots,
 //             with leading 1="Startbit" before the "dash/dot-matrix",
 //             dash/dot-matrix is in Bit0..max.Bit5: 
 //                  0=dot 1=dash
 //             Bit0 always contains the LAST TRANSMITTED dash/dot.
{
  BYTE bNrOfElements;

  if(bPattern.7)  // bit 7 set -> SPACE or CONTROL character
   { bPattern.7 = 0;   // reset space/control flag
     if(bPattern.6)    // bit6 of letter is control flag
      {   // bit7=1, bit6=1 bit5=x -> special functions ( 0b11Xn.nnnn )
         bPattern.6 = 0;
         if(bPattern.5) // 0b111X.nnnn = reserved for future use
          {
          }
         else           // 0b110n.nnnnn = extra long dash (with key down!)
          {             // max length = 31 (DASH TIMES)
           TxKeyDown();        // transmitter "key down"
           W = bPattern << 1;  // load bits0..5 of pause code into "dot counter"
           W += bPattern;      // dash time multiplied by THREE gives dot time
           SendTone(W);        // key down with sidetone for extra long DASH
           TxKeyUp();          // transmitter "key up"
          }
      } // end if    < 0b11Xn.nnnn >
     else // < 0b10nn.nnnn > -> start a "long pause" time, length coded in 5 bit
      {   // analyze 6-bit "pause" code (variable length, max = 63 (DOT TIMES)
        SendPause(bPattern);   // wait for long pause time
      } // end else  < 0b10Xn.nnnn >
   }
  else  // bit 7 not set -> normal character, neither SPACE nor CONTROL
   {    // play 'normal' letter :
     // Prepare transmission of next letter (in KY_char_latch):
     //  Shift left until the "start bit" is in the MSB, 
     //  and count the number of dots and dashes in the letter.
     // Examples:
     //    letter 'e':   bPattern = 0b00000010 (.)      
     //                -> shifted 5 times  -> 1 element 
     //    token 'sk':   bPattern = 0b01000101 (...-.-)
     //                -> shifted 0 times  -> 6 elements
     bNrOfElements = 6;    // load number of dots+dashes ...
     while( ! bPattern.6 )
      { bPattern <<= 1;
        bNrOfElements--;
        if(!bNrOfElements)
          break;
      }
     // Arrived here, bPattern holds the count of 'elements' in the pattern.
     // Bit 5 in bPattern holds the next element to be transmitted .
     while( bNrOfElements )
      {
        TxKeyDown();      // transmitter "key down"         
        // check if next element is a DASH or a DOT,
        // load timer with single or triple dot time.
        W = 1;            // guess it's a DOT
        if( bPattern.5 )
           W = 3;         // no, it's a DASH
        SendTone(W);      // wait one or three dot times
        TxKeyUp();        // transmitter "key up"
        SendPause(1);     // wait one dot time for gap within a letter
        bPattern <<= 1;   // move next DASH / DOT bit in
        bNrOfElements--;
      }
     SendPause(2);         // wait two more dot times for gap BETWEEN LETTERS
   } // end if <normal character to be transmitted>
} // end SendMorse()



//=======================================================================
// Read a single Byte from the PIC's Data-EEPROM.
//  Input parameters:
//     w  contains byte address
//
//  Result:
//     w  returns the read byte
//=======================================================================
BYTE ReadFromEEPROM(char W)
{

#ifdef __16F84
        EEADR = W;         // write into EEPROM address register
        GIE = 0;           // disable INTs
        RP0 = 1;           // Bank1 for "EECON" access
        RD  = 1;           // set "Read"-Flag for EEPROM
        RP0 = 0;           // normal access to Bank0
        GIE = 1;           // re-enable interrupts
        return EEDATA;     // read byte from EEPROM latch
#endif // __16F84
#ifdef __16F628
        // grrrrrrr ... EEDATA and EEADR have been moved from Bank0(16F84) to Bank1(16F628)
        //              and the rotten example from the datasheet telling you to switch to 
        //              bank0 to access EEDATA, which is rubbish (DS40300B page 93 example 13-1).
        // Caution: Under MPLAB 5.50, the EEPROM in the PIC16F628 is not emulated properly.
        GIE = 0;           // disable INTs
        RP0 = 1;           // Bank1 for ***ALL*** EEPROM registers in 16F628 (!!!)
        EEADR = W;         // write into EEPROM address register
        RD  = 1;           // set "Read"-Flag for EEPROM
                           //   why is EECON1.RD not cleared in MPLAB-sim ?!?
        W = EEDATA;        // read byte from EEPROM latch
        RP0 = 0;           // normal access to Bank0
        GIE = 1;           // re-enable interrupts
        return W;          // back to caller
#endif // __16F628
} // end ReadFromEEPROM




//----------------------------------------------------------------------------
void power_down(void)
  // The PIC will spend most of its lifetime in this routine
  // which switches to 'Power Down' state (using the sleep instruction).
  //
  // It returns to the caller as soon as "something happens" on Port B.
{  

  // Set "wakeup" interrupt source to "Change on Port B" only .
  // Note that CC5x supports binary values, other compiler's dont .
  // From datasheet:
  // > Four of the PORTB pins, RB7:RB4, have an interrupton-
  // > change feature. Only pins configured as inputs can
  // > cause this interrupt to occur (i.e., any RB7:RB4 pin
  // > configured as an output is excluded from the interrupton-
  // > change comparison). The input pins (of RB7:RB4)
  // > are compared with the old value latched on the last
  // > read of PORTB. The mismatch outputs of RB7:RB4
  // > are ORed together to generate the RB port change
  // > interrupt with flag bit RBIF (INTCON<0>).
  // >
  // > This interrupt can wake the device from Sleep. The
  // > user, in the Interrupt Service Routine, can clear the
  // > interrupt in the following manner:
  // > a) Any read or write of PORTB. This will end the
  // >    mismatch condition.
  // > b) Clear flag bit RBIF.
  // > A mismatch condition will continue to set flag bit RBIF.
  // > Reading PORTB will end the mismatch condition and
  // > allow flag bit RBIF to be cleared.
       W = PORTB;     // read PORTB to resolve 'mismatch'
       RBIF = 0;
  // > The interrupt-on-change feature is recommended for
  // > wake-up on key depression operation and operations
  // > where PORTB is only used for the interrupt-on-change
  // > feature. Polling of PORTB is not recommended while
  // > using the interrupt-on-change feature.
  // By WoBu: No interrupt service routine is required.
  //          If the interrupt is disabled, the device wakes up 
  //          and continues after the sleep instruction (see below)
       RBIE   = 0;            // we don't need an interrupt for this !
       INTCON = 0b00001000 ;  //  new INTCON value: only RB0 Interrupt
              //  ||||||||
              //  |||||||+--RBIF: RB Port Change Interrupt Flag 
              //  ||||||+---INTF: External Interrupt Flag 
              //  |||||+----T0IF: Timer0 Overflow Interrupt Flag 
              //  ||||+-----RBIE: Port Change Interrupt Enable
              //  |||+------INTE: INT Interrupt Enable
              //  ||+-------T0IE: T0IF Interupt Enable
              //  |+--------EEIE: EE Write Complete Interrupt Enable
              //  +---------GIE:  Global Interrupt Enable
       sleep();       // go to sleep, supply current less than 1 uA now..
       nop();
       W = PORTB;     // read PORTB to resolve 'mismatch'
       RBIF = 0;      // clear the possibly pending interrupt flag
       INTCON = 0x00; // disable interrupt on "Port B change"
} // end power_down()




//----------------------------------------------------------------------------
void main( void)
{
 uns8 b;

    VRCON = 0x00;          // turn voltage reference module off (16F628)
    CMCON = 0x07;          // turn comparators off and          (16F628)  
                           //   enable pins for digital I/O     (16F628)

    OSCF  = 0;    // PCON.OSCF = 0 -> 37kHz for internal RC oscillator

    // Initialize port directions etc ...
    // Port A is used as digital output for reasons explained below. 
    // Note: PORTA has no internal pull-ups. Unused INPUTS should be tied to Vcc
    //       with an external pull-up to avoid floating, which draws add'l current.
    //       To minimize the count of external components, port A is "output only".
    PORTA = 0x00; // Initialize the port by clearing output data latches
 // ADCON1= 0x06; // Configure all pins as digital inputs
    TRISA = 0x00; // Clear all bits in the TRIS register to make it an OUTPUT
    PORTA = 0x00; // set all outputs to LOW LEVEL (modulation, PTT, sidetone, thermo sensor)


    // Port B is used for digital inputs from paddle and message keys 
    // Note: PORTB will have the pullups enabled, so there is no need 
    //       to tie unused inputs to ground or Vdd .
    PORTB = 0x00; // provide a "save transition".. 
    TRISB = 0xFF; // only RB2 is an output (LED), the rest are INPUTS
    RBPU_ = 0;    // a bit in the OPTION register, zero means pullups ENABLED


    // Initialize application variables
    KY_mem_index = 0;  // start "playing" the beacon text beginning at first letter

    while(1)  // main loop during NORMAL OPERATION
     {
      b = ReadFromEEPROM(KY_mem_index);  // fetch next character (ASCII) from beacon message memory
      if(b)                              // zero-byte means end of the message string
       { KY_mem_index++;                 // next character index to read from
         b = Ascii_to_CW(b);             // convert ASCII into CW (Morse) code pattern
         SendMorse(b);                   // send in morse code
       }
      else  // got a ZERO byte from the message memory..
       {
         KY_mem_index = 0;               // start again with the 1st character
       } 
     } // end while(1)


} // end main()
